from pprint import pformat
from textwrap import indent

class OpenAddressHashTable:
    def __init__(self, size):
        self.size = size           # 固定長配列のサイズ
        self.keys = [None] * size  # 固定長配列
        self.vals = [None] * size  # 固定長配列

    def hash_index(self, key):
        h = hash(key)  # Python組み込みのハッシュ関数を利用
        return h % self.size

    def set(self, key, value):
        index = self.hash_index(key)
        print(
            f"{key}のindex={index}の空きを確認しkey={key},value={value}を格納")
        for shift in range(self.size):
            i = (index + shift) % self.size  # 位置ずらし
            if self.keys[i] is None:  # 衝突しなければ
                self.keys[i] = key    # i位置に格納
                self.vals[i] = value  # i位置に格納
                return  # 格納できたので終了
            elif self.keys[i] == key:  # 同じキーなら更新
                self.vals[i] = value  # i位置に格納
                return  # 格納できたので終了
            print(
                f"\t衝突発生: i={i}が使用中のため、次の"
                "インデックスの空きを確認")
        raise KeyError(
            "衝突発生: 空きインデックスが見つかりません")

    def get(self, key):
        index = self.hash_index(key)
        for shift in range(self.size):
            i = (index + shift) % self.size  # 位置ずらし
            if self.keys[i] == key:  # キーが一致するか？
                return self.vals[i]
            print(
                f"i={i}のキーが一致しないため、"
                "次のインデックスのキーを確認します")
        raise KeyError(key)  # 一致するキーがない

    def __str__(self):
        results = [
            (i, self.keys[i], self.vals[i])
            for i in range(self.size)
        ]
        s = indent(
            pformat(results, width=40),
            prefix='\t')
        return (
            f'{self.__class__.__name__}({self.size})=\n'
            + s
        )

table = OpenAddressHashTable(3)
table.set('名前', 'コーリー')
print(table)
table.set('住所', 'カリフォルニア')
print(table)
table.set('職業', 'プログラマー')
print(table)
table.set('年齢', 32)
print(table)
print(f"名前={table.get('名前')}")
print(f"年齢={table.get('年齢')}")