from pprint import pformat
from textwrap import indent

class Node:
    def __init__(self, key, value, next=None):
        self.key = key
        self.value = value
        self.next = next

class ChainHashTable:
    def __init__(self, size):
        self.size = size  # 固定長配列のサイズ
        self.table = [None] * size  # 固定長配列

    def hash_index(self, key):
        h = hash(key)  # Python組み込みのハッシュ関数を利用
        return h % self.size

    def set(self, key, value):
        index = self.hash_index(key)
        print(
            f"{key}のindex={index}にkey={key},value={value}を格納")
        if self.table[index] is None:
            node = Node(key, value)
            self.table[index] = node
        else:
            node = self.table[index]
            while node:
                if node.key == key:
                    # keyが格納済みならvalueを更新
                    node.value = value
                    return  # 格納できたので終了
                elif node.next is None:
                    # 次のノードがもうないなら末尾に格納
                    node.next = Node(key, value)
                    return  # 格納できたので終了
                # 次のノードを確認
                node = node.next

    def get(self, key):
        index = self.hash_index(key)
        node = self.table[index]
        while node:
            if node.key == key:  # キーが一致するか？
                return node.value
            node = node.next
        raise KeyError(key)  # 一致するキーがない

    def __str__(self):
        results = []
        for i in range(self.size):
            linked_list = []
            node = self.table[i]
            while node:
                linked_list.append((node.key, node.value))
                node = node.next
            results.append((i, linked_list))
        s = indent(
            pformat(results, width=55),
            prefix='\t')
        return (
            f'{self.__class__.__name__}({self.size})=\n'
            + s
        )

table = ChainHashTable(3)
table.set('名前', 'コーリー')
print(table)
table.set('住所', 'カリフォルニア')
print(table)
table.set('職業', 'プログラマー')
print(table)
table.set('年齢', 32)
print(table)