from collections import defaultdict

def directed_to_undirected(graph):
    ud_graph = defaultdict(dict)

    for vertex, neighbors in graph.items():
        for neighbor, weight in neighbors.items():
            ud_graph[vertex][neighbor] = weight
            ud_graph[neighbor][vertex] = weight

    return dict(ud_graph)

import heapq

def dijkstra(graph, starting_vertex):
    distances = {v: float('infinity') for v in graph}
    distances[starting_vertex] = 0
    pq = [(0, starting_vertex)]

    solved = {}  # 確定済みの経路だけ持たせる辞書
    step = 0  # ステップ数カウンター
    while len(pq) > 0:
        step += 1
        input('Enter')  # Enterキーで次のステップを実行
        print(f'ステップ: {step} 頂点の最短経路確定')
        print(f'\tPriQ: {sorted(pq)}')
        print(f'\t距離: {distances}')
        print('\t操作: キューの先頭から頂点を取り出し')
        cur_d, cur_v = heapq.heappop(pq)
        print(f'\t現在の頂点: {cur_v}, 距離={cur_d}')
        if cur_d > distances[cur_v]:
            print(f'\t訪問: {cur_v}(距離='
                  f'{distances[cur_v]})は訪問済み')
            continue
        else:
            print(f'\t訪問: {cur_v}(距離='
                  f'{cur_d})に最短経路で訪問する')
            solved[cur_v] = cur_d
            print(f'\t確定: {solved}')

        i = 0  # サブステップカウンター
        for neighbor, weight in graph[cur_v].items():
            i += 1
            input('Enter')  # Enterキーで次のステップを実行
            distance = cur_d + weight
            if distance < distances[neighbor]:
                distances[neighbor] = distance
                heapq.heappush(pq, (distance, neighbor))
                ope = (
                    f'{neighbor}の距離{distance}は短い'
                    'ので訪問予定キューに追加')
            else:
                ope = (
                    f'{neighbor}の距離{distance}は長い'
                    '経路なので再訪問しない')

            print(f'ステップ: {step}.{i} 隣接頂点の確認')
            print(f'\t現在の頂点: {cur_v}, 距離={cur_d}')
            print(f'\t隣接頂点: {neighbor}, 重み={weight}'
                  f', 距離={cur_d}+{weight}={distance}')
            print(f'\t操作: {ope}')
            print(f'\tPriQ: {sorted(pq)}')
            print(f'\t距離: {distances}')

    return distances

from pprint import pprint

graph = {
    'A': {'B': 2, 'C': 6},
    'B': {'D': 5},
    'C': {'D': 8},
    'D': {},
}

undirected_graph = directed_to_undirected(graph)
pprint(undirected_graph)
print('結果:', dijkstra(undirected_graph, "A"))
